import ctypes
from typing import List, Optional
from .__library__ import lib, get_error
from .camera import *
from .processing_config import *
from .camera_config import *


class KeaCamera(Camera):
    """
    The main interface to the kea camera
    """
    def __del__(self):
        if self._ptr:
            lib.tof_kea_camera_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, processing_config: Optional[ProcessingConfig] = None, serial: Optional[str] = None, camera_config: Optional[CameraConfig] = None, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new kea_camera class
        ----------

        Construct the kea camera

        Parameters
        ----------
        processing_config : ProcessingConfig
                The processing config
        serial : str
                The serial number of the camera to connect to, if the camera is
                available over both usb and gige it will be connected over usb. If
                an empty string is supplied the camera will connect to the first usb
                camera, if no usb camera is connected it will connect to the first
                gige camera that replies (this reply time is random, so you might
                connect to a different camera each time if you've got multiple
                cameras connected to your network). If no camera is found with the
                given serial number a nullptr and error is returned.
        ----------

        Construct the kea camera with a processing config generated by
        default_processing from the camera config

        Parameters
        ----------
        serial : str
                The serial number of the camera to connect to, if the camera is
                available over both usb and gige it will be connected over usb. If
                an empty string is supplied the camera will connect to the first usb
                camera, if no usb camera is connected it will connect to the first
                gige camera that replies (this reply time is random, so you might
                connect to a different camera each time if you've got multiple
                cameras connected to your network). If no camera is found with the
                given serial number a nullptr and error is returned.
        ----------

        Construct the kea camera with the given processing and camera config.

        Parameters
        ----------
        processing_config : ProcessingConfig
                The processing config
        serial : str
                The serial number of the camera to connect to, if the camera is
                available over both usb and gige it will be connected over usb. If
                an empty string is supplied the camera will connect to the first usb
                camera, if no usb camera is connected it will connect to the first
                gige camera that replies (this reply time is random, so you might
                connect to a different camera each time if you've got multiple
                cameras connected to your network). If no camera is found with the
                given serial number a nullptr and error is returned.
        camera_config : CameraConfig
                The camera config
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        elif processing_config != None and serial != None:
            self._ptr = lib.tof_kea_camera_new(processing_config._ptr, serial.encode("utf-8"), get_error())
        elif serial != None:
            self._ptr = lib.tof_kea_camera_new_simple(serial.encode("utf-8"), get_error())
        elif processing_config != None and serial != None and camera_config != None:
            self._ptr = lib.tof_kea_camera_new_both_configs(processing_config._ptr, serial.encode("utf-8"), camera_config._ptr, get_error())
        else:
            raise RuntimeError("Can't construct empty class")

    def onCameraProcessingCapable(self) -> bool:
        """
        Check whether this kea camera can do on camera processing

        Returns
        ----------
        bool
            On camera processing capable
        """
        return lib.tof_kea_camera_on_camera_processing_capable(self._ptr, get_error())


    def getOnCameraProcessing(self) -> bool:
        """
        Get on camera processing

        Returns
        ----------
        bool
            True on camera processing, false on host processing
        """
        return lib.tof_kea_camera_get_on_camera_processing(self._ptr, get_error())


    def setOnCameraProcessing(self, on_camera_processing: bool) -> None:
        """
        Set on camera processing

        Parameters
        ----------
        on_camera_processing : bool
            True on camera processing, false on host processing
        """
        return lib.tof_kea_camera_set_on_camera_processing(self._ptr, on_camera_processing, get_error())


    def getDelay(self) -> int:
        """
        Get delay between network packets

        Returns
        ----------
        int
            The delay, depending on the camera software it is in nanoseconds or in
            cpu instruction
        """
        return lib.tof_kea_camera_get_delay(self._ptr, get_error())


    def setDelay(self, delay: int) -> None:
        """
        Set delay between network packets

        Parameters
        ----------
        delay : int
            The delay, depending on the camera software it is in nanoseconds or in
            cpu instruction
        """
        return lib.tof_kea_camera_set_delay(self._ptr, delay, get_error())


    def getPacketSize(self) -> int:
        """
        Get maximum size of each network packet transmitted

        Returns
        ----------
        int
            Packet size in bytes
        """
        return lib.tof_kea_camera_get_packet_size(self._ptr, get_error())


    def setPacketSize(self, size: int) -> None:
        """
        Set maximum size of each network packet transmitted

        Parameters
        ----------
        size : int
            Packet size in bytes
        """
        return lib.tof_kea_camera_set_packet_size(self._ptr, size, get_error())


    def setCameraConfig(self, camera_config: CameraConfig) -> None:
        """
        Set the camera config

        Parameters
        ----------
        camera_config : CameraConfig
            Camera config
        """
        return lib.tof_kea_camera_set_camera_config(self._ptr, camera_config._ptr, get_error())


    def setConfigurations(self, camera_config: CameraConfig, processing_config: ProcessingConfig) -> None:
        """
        Set both the camera and processing config

        Parameters
        ----------
        camera_config : CameraConfig
            Camera config
        processing_config : ProcessingConfig
            Processing config
        """
        return lib.tof_kea_camera_set_configurations(self._ptr, camera_config._ptr, processing_config._ptr, get_error())


    def version(self) -> str:
        """
        Get the tof library version running on the camera

        Returns
        ----------
        str
            Version
        """
        return lib.tof_kea_camera_version(self._ptr, get_error()).decode("utf-8")


    def softwareTriggerCapable(self) -> bool:
        """
        Check whether the camera is capable of software trigger

        Returns
        ----------
        bool
            Software trigger capable
        """
        return lib.tof_kea_camera_software_trigger_capable(self._ptr, get_error())


    def softwareTrigger(self) -> None:
        """
        Software trigger the camera
        """
        return lib.tof_kea_camera_software_trigger(self._ptr, get_error())


    def remoteUpdateCapable(self) -> bool:
        """
        Check whether the camera is capable of updating remotely

        Returns
        ----------
        bool
            Remote update capable
        """
        return lib.tof_kea_camera_remote_update_capable(self._ptr, get_error())


    def remoteUpdate(self, file_location: str) -> None:
        """
        Update the camera remotely. After uploading the update file the function
        will return. If a correct update file was uploaded, the camera will flash
        the update and reboot. This generally takes about a minute. The camera can
        also be updated via a web interface by visiting the ip address of the camera
        on port 8080, for example http://192.168.1.208:8080

        Parameters
        ----------
        file_location : str
            Location of the update file
        """
        return lib.tof_kea_camera_remote_update(self._ptr, file_location.encode("utf-8"), get_error())


class DiscoveredKea():
    """
    Simple class containing information about found kea cameras
    """
    def __del__(self):
        if self._ptr:
            lib.tof_discovered_kea_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new discovered_kea class
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        else:
            raise RuntimeError("Can't construct empty class")

    def serial(self) -> str:
        """
        The serial

        Returns
        ----------
        str
            Serial
        """
        return lib.tof_discovered_kea_serial(self._ptr, get_error()).decode("utf-8")


    def info(self) -> str:
        """
        Additional information about the discovered camera

        Returns
        ----------
        str
            Additional information
        """
        return lib.tof_discovered_kea_info(self._ptr, get_error()).decode("utf-8")


def discoverKeaCameras() -> List[DiscoveredKea]:
    """
    Discover all available kea cameras

    Returns
    ----------
    List[DiscoveredKea]
        All available kea cameras
    """
    size = lib.tof_discover_kea_cameras(ctypes.c_void_p(), 0, get_error())
    array = (ctypes.c_void_p * size)()
    lib.tof_discover_kea_cameras(array, size, get_error())
    return [DiscoveredKea(ptr = item) for item in array]

def createKeaCameraGige(processing_config: ProcessingConfig, serial: str, packet_size: int = 1472) -> KeaCamera:
    """
    Construct a kea camera using the gige interface. This could be useful in two
    cases. The first is when the camera is connected with both usb and ethernet but
    you want to use ethernet. The second is when for some reason the default maximum
    packet size is too large.

    Parameters
    ----------
    processing_config : ProcessingConfig
        The processing config
    serial : str
        The serial number of the camera to connect to, if the camera is available
        over both usb and gige it will be connected over usb. If an empty string is
        supplied the camera will connect to the first usb camera, if no usb camera
        is connected it will connect to the first gige camera that replies (this
        reply time is random, so you might connect to a different camera each time
        if you've got multiple cameras connected to your network). If no camera is
        found with the given serial number a nullptr and error is returned.
    packet_size : int
        The packet size to use for communication over ethernet

    Returns
    ----------
    KeaCamera
        A kea camera
    """
    return KeaCamera(ptr = lib.tof_create_kea_camera_gige(processing_config._ptr, serial.encode("utf-8"), packet_size, get_error()))


